/*
** File: model_admxrc7v1.c
** Project: ADMXRC3 module driver
** Purpose: ADM-XRC-7V1 specific code, including the function to bootstrap
**          the device context.
**
** (C) Copyright Alpha Data 2012
*/

#include <df.h>

#include "device.h"
#include "info.h"
#include "model_boot.h"
#include <core/admxrc7v1.h>

static ADMXRC3_STATUS
getBankInfo(
	Admxrc3DeviceContext* pDevCtx,
  unsigned int bankIndex,
  const void* pVpdBuffer,
  IOCTLS_ADMXRC3_GETBANKINFO* pIoctl)
{
  VpdAdmxrc7v1Rev0* pVpd = (VpdAdmxrc7v1Rev0*)pVpdBuffer;
  uint8_t order;

  if (bankIndex >= DF_ARRAY_LENGTH(pVpd->sdram)) {
    return ADMXRC3_INVALID_INDEX;
  }
  order = pVpd->sdram[bankIndex].order;
  pIoctl->out.typeMask = ADMXRC3_BANK_SDRAM_DDR3;
  pIoctl->out.physicalDataWidth = 32;
  pIoctl->out.physicalEccWidth = 0;
  pIoctl->out.maximumFrequency = (uint64_t)pVpd->sdram[bankIndex].speed * 100000;
  pIoctl->out.minimumFrequency = 303030303U;
  pIoctl->out.connectivityMask = 0x1U;
  pIoctl->out.physicalSize = (uint64_t)0x1U << order;
  pIoctl->out.bPresent = (uint8_t)(order != 0);

  return ADMXRC3_SUCCESS;
}

static void
getCardInfo(
	Admxrc3DeviceContext* pDevCtx,
  const void* pVpdBuffer,
  IOCTLS_ADMXRC3_GETCARDINFO* pIoctl)
{
  VpdAdmxrc7v1Rev0* pVpd = (VpdAdmxrc7v1Rev0*)pVpdBuffer;
  unsigned int i;

  pIoctl->out.serialNumber = pVpd->serial;
  pIoctl->out.numMemoryBank = 4;
  pIoctl->out.memoryBankPresent = 0;
  for (i = 0; i < DF_ARRAY_LENGTH(pVpd->sdram); i++) {
    if (0 != pVpd->sdram[i].order) {
      /* Bank is fitted */
      pIoctl->out.memoryBankPresent |= (0x1U << i);
    }
  }
}

static ADMXRC3_STATUS
getFpgaInfo(
	Admxrc3DeviceContext* pDevCtx,
  unsigned int targetIndex,
  const void* pVpdBuffer,
  IOCTLS_ADMXRC3_GETFPGAINFO* pIoctl)
{
  VpdAdmxrc7v1Rev0* pVpd = (VpdAdmxrc7v1Rev0*)pVpdBuffer;
  ADMXRC3_PACKAGE_TYPE package;
  ADMXRC3_FPGA_TYPE fpgaType;
  char speed[8];
  char stepping[8];

  switch (targetIndex) {
  case 0:
    fpgaType = (ADMXRC3_FPGA_TYPE)pVpd->fpga.device;
    break;

  default:
    return ADMXRC3_INVALID_INDEX;
  }

  switch (fpgaType) {
  case ADMXRC3_FPGA_7V1500T:
    package = ADMXRC3_PACKAGE_MAKEFL(1761);
    break;

  case ADMXRC3_FPGA_7V2000T:
    package = ADMXRC3_PACKAGE_MAKEFH(1761);
    break;

  default:
    package = ADMXRC3_PACKAGE_MAKEFF(1761);
    break;
  }

  dfAssert(sizeof(char) == sizeof(uint8_t));

  dfStrCpy(speed, DF_ARRAY_LENGTH(speed), (const char*)pVpd->fpga.speedGrade);
  switch (pVpd->fpga.tempGrade) {
  case ADMXRC7V1_VPD_TEMPGRADE_COMMERCIAL:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "C");
    break;

  case ADMXRC7V1_VPD_TEMPGRADE_INDUSTRIAL:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "I");
    break;

  case ADMXRC7V1_VPD_TEMPGRADE_EXTENDED:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "E");
    break;

  default:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "?");
    break;
  }

  dfStrCpy(stepping, DF_ARRAY_LENGTH(stepping), (const char*)pVpd->fpga.stepping);
  pIoctl->out.flags = ADMXRC3_FPGA_SPEEDVALID;
  if (stepping[0] != '\0') {
    pIoctl->out.flags |= ADMXRC3_FPGA_STEPPINGVALID;
  }
  setupIoctlsGetFpgaInfo(pDevCtx, pIoctl, fpgaType, package, speed, stepping, TRUE);

  return ADMXRC3_SUCCESS;
}

static void
bootstrap(
	Admxrc3DeviceContext* pDevCtx)
{
  pDevCtx->methods.pGetBankInfo = getBankInfo;
  pDevCtx->methods.pGetCardInfo = getCardInfo;
  pDevCtx->methods.pGetFpgaInfo = getFpgaInfo;
  pDevCtx->bReverseSelectMapData = FALSE;
  pDevCtx->localAddressLimit = (uint64_t)(0x1ULL << 39U);
  pDevCtx->localWidth = 16U;
}

void
admxrc3BootstrapAdmxrc7v1(
	Admxrc3DeviceContext* pDevCtx)
{
  dfDebugPrint(1, ("admxrc3BootstrapAdmxrc7v1: entered\n"));
  bootstrap(pDevCtx);
}

void
admxrc3BootstrapAdmvpx37v2(
	Admxrc3DeviceContext* pDevCtx)
{
  dfDebugPrint(1, ("admxrc3BootstrapAdmvpx37v2: entered\n"));
  bootstrap(pDevCtx);
}
